#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <signal.h>

#include "log.h"
#include "udpcast.h"
#include "udp-receiver.h"
#include "socklib.h"
#include "version.h"

#define MAXBLOCK 1024

static struct option options[] = {
    { "file", 1, NULL, 'f' },
    { "pipe", 1, NULL, 'p' },
    { "port", 1, NULL, 'P' },
    { "portbase", 1, NULL, 'P' },
    { "interface", 1, NULL, 'i' },
    { "ttl", 1, NULL, 0x0401 },
    { "mcast_all_addr", 1, NULL, 'M' },
    { "mcast-all-addr", 1, NULL, 'M' },

#ifdef LOSSTEST
    /* simulating packet loss */
    { "write-loss", 1, NULL, 0x601 },
    { "read-loss", 1, NULL, 0x602 },
    { "seed", 1, NULL, 0x603 },
    { "print-seed", 0, NULL, 0x604 },
    { "read-swap", 1, NULL, 0x605 },
#endif

    { "passive", 0, NULL, 0x0801 },

    { "nosync", 0, NULL, 0x0901 },

    { "log", 1, NULL, 'l' },
};

void intHandler(int nr) {
    fatal(1, "Cancelled by user\n");
}

void usage(char *progname) {
    fprintf(stderr, "%s [--file file] [--pipe pipe] [--portbase portbase] [--interface net-interface] [--log file] [--ttl time-to-live] [--mcast-all-addr mcast-all-address]\n", 
	    progname);
    exit(1);
}

int main(int argc, char **argv)
{
    char *ptr;
    struct net_config net_config;
    struct disk_config disk_config;
    int c;
    int doWarn=0;

#ifdef LOSSTEST
    int seedSet = 0;
    int printSeed = 0;
#endif

    disk_config.fileName=NULL;
    disk_config.pipeName=NULL;
    disk_config.flags = 0;

    net_config.portBase = 9000;
    net_config.rateLimit = NULL;
    net_config.ttl = 1;
    net_config.flags = 0;
    net_config.mcastAll = NULL;

    ptr = strrchr(argv[0], '/');
    if(!ptr)
	ptr = argv[0];
    else
	ptr++;
    
    net_config.net_if = getenv("IFNAME");
    if (!net_config.net_if)
	net_config.net_if = "eth0";
    if (strcmp(ptr, "init") == 0) {
	doWarn = 1;
	disk_config.pipeName = strdup("/bin/gzip -dc");
	disk_config.fileName = "/dev/hda";
    }
    while( (c=getopt_long(argc, argv, "f:p:P:i:l:M:",
			  options, NULL)) != EOF ) {
	switch(c) {
	    case 'f':
		disk_config.fileName=optarg;
		break;
	    case 'i':
		net_config.net_if=optarg;
		break;
	    case 'p':
		disk_config.pipeName=optarg;
		break;
	    case 'P':
		net_config.portBase = atoi(optarg);
		break;
	    case 'l':
		log = fopen(optarg, "a");
		break;
	    case 0x0401:
		net_config.ttl = atoi(optarg);
		break;
	    case 'M':
		net_config.mcastAll = strdup(optarg);
		break;
#ifdef LOSSTEST
	    case 0x601:
		setWriteLoss(optarg);
		break;
	    case 0x602:
		setReadLoss(optarg);
		break;
	    case 0x603:
		seedSet=1;
		srandom(strtoul(optarg,0,0));
		break;
	    case 0x604:
		printSeed=1;
		break;
	    case 0x605:
		setReadSwap(optarg);
		break;
#endif
	    case 0x801:
		net_config.flags|=FLAG_PASSIVE;
		break;
	    case 0x901:
		disk_config.flags|=FLAG_NOSYNC;
		break;
	    case '?':
		usage(argv[0]);
	}
    }

    fprintf(stderr, "Udp-receiver %s\n", version);

#ifdef LOSSTEST
    if(!seedSet)
	srandomTime(printSeed);
#endif

    signal(SIGINT, intHandler);
    startReceiver(doWarn, &disk_config, &net_config);
    return 0;
}
